/* -----------------------------------------------------------------------

                               MAIN3.PRG

    This is the main control program. It sets switches and calls everyone
    else.

    Version copied from main.prg to do JPE revisions. Main revisions

    1) calculates prices of stocks with dividends /= consumption, perpetuity.
    2) does not track region with lambda = 0, for better match to cont time
      model. _sg is limited below sbar; beyond uses linear interpolation.
      (ALso allows one to chop the state vector at sbar, but this option
       is not debugged.)
    3) calculates annualized artificial data for any simulation interval
    4) calculates risk aversion , internal marginal utility
    5) translated into new paper notation (relative main2.prg) s not z.

    All procs are contained in mainprc.prg, and referenced via library
    \j\camrb\newprogs\camrb.lcg.

    There are three main procs; a solution uses each in order. 1) findlpd
    finds p/d etc. on grid. 2) finders finds E(r), etc. on grid 3) simvars
    makes artificial data at simulation frequency, annvars annualizes them.

    How to use this:
    Everything is controlled by switches in the beginning of this program.
    Set the switches as you want them then go. Then run follow-on
    programs such as graphs.prg, tables.prg.

    Hints:
    To try something new, use few gridpoints (sdivs option), a coarse
    convergence criterion (convtol option), few simulations (_ncalc option)
    and a coarse simulation interval (_tsc = 1 or 4 not 12). When it works
    go back to the more time-intensive and precise settings.


    NOTE: since Gauss' numerical integrator and optmization routines do not
         allow multiple arguments, there are a bunch of globals that are passed
         to these procs.  They are all of the form _xxx.
         be careful when you reset these variables. Also make sure that
         globals are set right before each proc is called

   naming conventions for prices, returns, etc.
     no suffix indicates a function of state (lnr, _lnpd)
   t suffix indicates a time series (lnrt)
   t alone = annualized simulated data   (lnrt)
   tb      = raw simulated data (note funcitons of state are 1 longer here)
   ta      = long annual data
   tp      = postwar annual data
   tq      = postwar quarterly data

Things to do (maybe).
   1) add code to make sure all prices, not just dividend claim, are settling
      down. Danger is they add 1% per month,e.g. 0.01 in logs so creep up\
      and you don't notice it.
   2) do a good job of dividend growth calibration, see if it does enough
      better on following statistics that paper tables should reflect that
      higher gamma rather than same gamma.
   3) reprogram annvars and simvars so that you can do really long simulation
      without running out of memory. Or use virtual.
   4) reprogram internal habit with a better transformation than ln(S) to approx
      imate marginal utility. Then
      numerical integrator can do a better job of fitting the wild
      tail behavior. Is the departure from condidtional mean variance efficiency
      real or due to bad integrating?
   5) In feeding real time series to model, I split growth over 12 quarters.
     This does not produce a monthly time series that exactly reaggregates (by
      summing the level over 12 months of year) to the original annual
       time series. Find a way to disaggregate that does reaggregate but
      still makes a pretty picture.

-------------------------------------------------------------------------- */


library pgraph   @,\j\camrb\newprogs\camrb@;
output file=main.out reset;

/* ------------------------------- */
/* clear globals created by procs  */
/* (avoids error messages)         */
/* ------------------------------- */

/* functions of state */

clearg _s,_sg, _s0, _lnpd,_lnpda,_lnpdx,_lnpd1,_lnpcd,
       _lnppa,_lnpp,_lnpdb,_stock,
       _lnzofs,_lnfmu,_lnpdint,_lnpsc;

clearg  er,erd,erc,elnr,elnrdb,sdr,
             sdrd,sdrc,sdlnr,sdlnrdb,
              lnrf,lnrf1,lny,lny2b,slpmv,shpint;

/* simvars globals */

 clearg  stb,lndctb,lnpdtb,lnpd1tb,lnpdctb,
         lnrtb,lnrdtb,lnrctb,
         ertb,erdtb,erctb,elnrtb,
         sdrtb,sdrdtb,sdrctb,sdlnrtb,
         lnrftb,elnrdbtb,lnytb,lnrdbtb,
         wtb,wctb;

 /* ann vars globals */

 clearg  lndct,lnpdt,lnpd1t,lnpdct,lnrt,lnrdt,lnrct,
         lnrft,lnyt,lnrdbt,wtb,wctb,
         lnctb,lnct,
         lnddt,lndt,lndtb,lnddtb,
         lnchpt,lnchptb,lnchpdt,lnchpdtb,st;


/* ----------------------- */
/* program running options */
/* ----------------------- */

srch = 0;           /* search for parameters or do one set */
                    /* see if srch below in this file for effect */

_intrnl = 0;  /* flag to use internal marg. util, then external and compare */
              /* see if ... and _intrnl below in this file for effect      */

_limits = 0;  /* forces s < smax in strans proc. Only affects things that
                 use that proc.  Useful, e.g. to keep cond. sd. positive
                 causes problems however. Refine it if you use it.          */

_tsc = 12;     /*  chooses time interval for simulation.                     */
              /* 1 = annual; 4 = quarterly, 12 monthly.                     */
              /* 12 is the standard.                                        */

_calpwa = 1;  /* choose parameter calibration                               */
_calpwq = 0;  /* pw annual, pw quarterly, or long term calibration          */
_callha = 0;  /* pwa is main calibration for paper                         */
_calc   = 1;  /* choose calibration to consumption claim or dividend        */
_cald   = 0;  /* cald simply uses a higher gamma to nail sharpe ratio. else=*/

_ncalc =  nd; @50000@;   /* Length of simulation in months.   */
_nredo = 1@0@;
                 /* number of times to redo simulation for sample statistics */
                 /* too big ncalc runs out of memory. statistics are averaged
                    _nredo times. */

sdivs = 12;      /* # of s gridpoints to use. 10 is standard */

_convtol = 0.0001;
                  /* max diff in p/d from diff starting vals to say conver.*/
                  /* set to zero if you want a fixed number of iterations,*/
                  /* e.g. 150 years */
_maxit = 400;     /* max years to iterate PD = E(M DelD (1+PD));  */
                  /* multiplied by _tsc below                     */

seedval = iseed;       /* random number seed. */
_prniter = 0;        /* print information on each iteration, graph p/d(s)  */
/*_uselast =;*/        /* use last p/d to start numerical? very fast, use   */
                     /* if you just want a new simulation or to get going  */
                     /* in the morning from where you last were            */

titles = 1;          /* print titles on graphs? 1 for overheads, 0 for paper*/
                     /* not yet implemented in graphs.prg                   */

_intlim   = 8|-8;    /* # of sd on each side for integrals. at least 6     */
_intord = 40;        /* how many points numerical integrator will use.     */
                     /* see help for intquad1. seems to need 40            */

                    /* which assets to compute prices   */
          /* cons claim   div  d-c   perp. bonds  S*C for risk av.  */
_assets =       1       |  1  | 0  |  0  |  0    |    0            ;

if _intrnl;
    _assets =   1       |  1  | 0  |  0  |  0    |    0            ;
endif;


_condtim = 0;     /* calcuate time series of conditional mean, variance....*/
                  /* 0 lets you save space in long simulations. Not currently */
                  /* used in any tables or graphs anyway.                 */

_savhist = 0 ;    /* calculate and save time series that use historical   */
                  /* data. 1 in order to produce graph of history         */
                  /* does two simulations with different paramters. See   */
                  /* if _savhist below in this file for effect.           */
                  /* Directions: 1: set _savhist=1 and _callha to do simu */
                  /* lation with long ts c data and params. Then do with  */
                  /* _savhist=1 and _calpwa to use pw data and params    */
                  /* then run histgraphs.prg to printout graph.              */

        /* Bond parameters. Left in here, but abandoned bonds a while ago. */
       /*  may still work. May not. Leave b=0 and won't bother you         */

_maxdb = 100;      /* max maturity of term structure. MUST be >= 2         */
                    /* in years                                             */
bondsel = 1|2|5|7|100;/* what bond maturities to look at in plot, tables    */
                     /* maturities in years */

_b = 0;            /* rf = _lnrf0 + _b(z-n).            */
                   /* -1 will find b to set rf(zmin) = 0 */

/* -------------- */
/* set parameters */
/* -------------- */


/* values for quarterly postwar data */

if  _calpwq;                      /* choose pwq parameters */

    _gam =  2.37198;
    _phi =   0.87^(1/_tsc);       /* use annual pd ac to calibrate */

    _bigg  = exp(0.0047*4/_tsc);  /* mean log consumption growth */
    _sigv  = 0.0056*2/_tsc^0.5;   /* s dev of log consumption growth */
    _lnrf0 = 0.0094/_tsc;         /* desired constant rf, will find delta */
endif;

/* values for annual postwar data. Current parameters for most simulations */

if  _calpwa;                       /* pw annual parameters */
     if _calc;                     /* calibrate consumption claim=stock */
        _gam = gamacc@1.89@;
    endif;
     if _cald;                     /* calibrate dividend growth*/
        if _tsc == 12;
            _gam =   4.8;
        endif;
        if _tsc == 1;
            _gam = 3.0;
        endif;
        if _tsc == 4;
            _gam = 4.0;
        endif;
    endif;
    _phi =   ficc^(1/_tsc);
    _bigg  = exp(gcc/_tsc); /* mean log consumption growth */
    _sigv  = 1.224/1.204*1.21*0.0122/_tsc^0.5;_sigv=csigcc/_tsc^0.5;
                                 /* s dev of log consumption growth */
                                /* first term accounts for time averaging */
    _lnrf0 = ln(rcc)/_tsc;      /* specify constant rf, will find delta */
endif;

                    /* long  annual parameters */
if  _callha;
     if _calc;     /* consumption claim */
        _gam =   0.7;
        _phi =   0.79^(1/_tsc);
        _bigg  = exp(0.0172/_tsc);  /* mean log consumption growth */
        _sigv  = 0.0332/_tsc^0.5;   /* s dev of log consumption growth */
        _lnrf0 = 0.01/_tsc;         /* specify constant rf, will find delta */
    endif;
endif;

/* common to all calibrations */

_sigw =  dsigcc/_tsc^0.5;          /* parameters for dividend growth  */
_rhow = rocc;                       /* correlation between c and div growth */

/*                              /* option to set Dd/Dc regression coeff to 1 */
_rhow = _sigv/_sigw;
"Using rho from sigv/sigw = " _rhow;
*/
_sigwc = 0.18;              /* std dev of d-c for stock model 2 */
                            /* does not change with time scale! */

/* ------------------ */
/* derived parameters */
/* ------------------ */

_bigsb = _sigv*(_gam/(1-_phi))^0.5;                        /* S bar */

if _b == -1;
     term = (_gam*(1-_phi)-_gam^2*_sigv^2+2*_lnrf0);
     disc = term^2 - 8*_gam*(1-_phi)*_lnrf0;
     if disc < 0; " discriminant < 0 " ; endif;
     _b = (term - disc^(1/2))/2;  /* the root that picks 0=b if rf=0 */
      "b to set rf(smax) = 0" _b;
     _bigsb = ((_gam^2*_sigv^2)/(_gam*(1-_phi)-_b))^(1/2);
endif;

_sbar = ln(_bigsb);                                        /* s bar */
_smax = _sbar + 1/2*(1-_bigsb^2);                          /* s max */
_delt = exp(_gam*ln(_bigg) - 1/2*_gam*(1-_phi) -_lnrf0);

if _b == 0;
   _maxdb=2;    /* no point in doing a big term structure */
endif;
bondsel = selif(bondsel,bondsel .<= _maxdb);


/* ------------------------------ */
/* include procedure definitions. */
/* ------------------------------ */

#include mainprc.prg;

"Doing main program";

        {_sg,_lnpda,_lnpd,_lnppa,_lnpp,_lnpdx,_lnpd1,_lnpcd,ctrindx,_lnpsc}=
                  findlpd(_gam,_bigsb,_phi,_b,_bigg,_sigv,_lnrf0,
                          _sigw,_rhow,_sigwc,sdivs,_smax);
