/* ======================================================================= */
/*                     procedure definitions                               */
/*                           for main3.prg                                 */
/* ======================================================================= */

/* --------------------------------- */
/* proc lamofs(s)                    */
/* conditional variance proc         */
/* returns lambda(s)                 */
/* --------------------------------- */

proc(1) = lamofs(s);
    local term;
    term = 1-2*(s-_sbar);
    term = (term .> 0).*term ;  /* some s are greater than smax where lamda=0*/
    term = 1/_bigsb*term^(1/2)-1;
    term = ( s .<= _smax).* term;
    retp(term);
endp;

  /* ---------------------------------------- */
  /* proc lamprim(s)                          */
  /* returns -1/{Sbar root (1-2(s-sb))}       */
  /* this is d lambda / d s not d lambda / dS */
  /* ---------------------------------------- */

proc(1) = lamprim(s);
    local term,i,j;
    term = 1-2*(s-_sbar);
    i = 1;
        do while i <= rows(s);
        j = 1;
            do while j <= cols(s);

            if s[i,j] <= _smax;    /* root is defined up to sbar+1/2 > smax,*/
                                   /* but lambda = 0 above smax */

           /*
           if s[i,j] < _sbar+1/2; /* above this, root is negative. */
           */
                term[i,j] = -1/(_bigsb*term[i,j]^(1/2));
            else;
                term[i,j] = 0;     /* report 0 slope wher lam = 0 */
            endif;
            j = j+1;
        endo;
        i = i+1;
    endo;
    retp(term);
 endp;


/* ----------------------------------------------------- */
/*                        proc findlpd                   */
/* Sets up s grid, finds ln p/d etc.  numerically        */
/* takes globals _gam,_phi,_bigg,_sigv,_lnrf0,..         */
/* creates globals _sg,_lnpda,_lnpd,_lnppa,_lnpp...      */
/* these HAVE TO be globals to be passed to int procs    */
/* they are only passed as parameters to remind you     */
/*
   _intrnl switch tells it whether to evaluate asset prices
    using internal vs. external marginal utility.

   Syntax:

    {sg,_lnpda,_lnpd,_lnppa,_lnpp,_lnpdx,_lnpd1,_lnpcd,ctrindx,_lnpsc}=
            findlpd(_gam,_bigsb,_phi,_b,_bigg,_sigv,_lnrf0,
                    _sigw,_rhow,_sigwc,sdivs,_smax);
*/
/* ----------------------------------------------------- */

   /* ------------------------------------- */
   /* subproc testit tests for convergence */
   /* ------------------------------------- */

    proc(2) = testit(newlpd,oldlpd);     /* finds  max % ch */
        local tv,tind,testval;
        tv = (exp(newlpd)-exp(oldlpd))./ exp(newlpd) ;
        tind = maxindc( abs(tv) );
        tv = tv[tind];
        retp(tv,tind);
    endp;

    /* ---------------------------- */
    /* subproc makgrid sets up grid */
    /* ---------------------------- */

 proc(1) = makgrid(_smax,_sbar,sdivs);
     clearg _sg;

    _sg = seqb(0,exp(_smax)*1,(exp(_smax)*1)/(sdivs+1));
                                                   /* *1: some room past smax */
                                                   /* even divs in S space */
    _sg = ln(_sg[2:rows(_sg)]);                /* no 0 so log doesn't blow up */

    if maxc(_sg .== _sbar) == 0;         /* make sure sbar is grid point*/
        _sg = _sg|_sbar;
    endif;
    if maxc(_sg .== _smax) == 0;          /* make sure smax is grid pt */
        _sg = _sg|(_smax);
    endif;
    _sg = _sg|(_smax - seqb(0.01,0.04,0.01));
                                           /* extra density near max */
                                           /* e.g. keeps E/s below mvf! */
/*
    _sg = _sg|ln(0.0005|0.001|.005);       /* extra way low improves E(mR)
                                               in artificial data */
*/
    _sg = sortc(_sg,1);
    retp(_sg);
endp;


/* ------------------ */
/* main proc findlpd  */
/* ------------------ */

proc(10) =  findlpd(_gam,_bigsb,_phi,_b,_bigg,_sigv,_lnrf0,
                    _sigw,_rhow,_sigwc,sdivs,_smax);

    local pdhist,lambda,lamprime,iter,i,
          delstr,aj,phij,j,ctrindx,
          test,testv,testind,testval,delprm,a2j,newlpp,
          newlpd,newlpdx,newlpd1, newlpcd,px,py,newlnz,lnzhist, newlpsc;

    /* ---------------- */
    /* Echo assumptions */
    /* ---------------- */

     format /RD 10,3;
    "       Sbar sbar"  _bigsb _sbar ;
    "      Smax smax"  exp(_smax) _smax;
    " delt ann.%delt"  _delt -_tsc*ln(_delt);
    "gamma phi phi^t"  _gam _phi _phi^_tsc;

    /* warnings put in when playing with dividend models. reminds you
       to check that for rho > 0, dividends really are converging!
       watch with _prninter=1 and make sure not growing 1% per year */

    if exp(_lnrf0) < _bigg;
    "Warning: annual interest rate"
               100*_lnrf0*_tsc " < growth rate " 100*ln(_bigg)*_tsc ;
      "   Risk neutral claim to consumption has infinite price";
    endif;

    if exp(_lnrf0) < _bigg*exp(1/2*_sigw^2);
      "Warning: annual interest rate"
               100*_lnrf0*_tsc " < Ge^1/2s^2 "
                100*_tsc*(ln(_bigg)+1/2*_sigw^2) ;
      "   claim to dividends uncorrelated with consumpiton has infinite price";
    endif;

     /* ----------- */
     /* set up grid */
     /* ----------- */

     _sg = makgrid(_smax,_sbar,sdivs);

    /* find index of steady state sbar. used to report statistics below */

    ctrindx = indexcat(_sg,_sbar+(-0.0001|0.0001));
    if ismiss(ctrindx);
        "error: ctrindx is missing value ";
    endif;

     /* ------------------------------------------ */
     /* find internal marginal utility if required */
     /* ------------------------------------------ */

    if _intrnl;

        /* ------------------------------------------- */
         /* Step 1: either load last time's or find Z(s) */
         /* ------------------------------------------- */

        if _uselast;
           load _lnzofs;

           /*                 cool graph, but not used in paper so deleted*/
           /*
           graphjc;
           title("lnZ(s)");
           px = _sg;
           py = _lnzofs;
           _plctrl = 1;
           _pstype = 4;
           _ptek = "lnzofs.tkf";
           xy(px,py);
            */
        endif;

        if (not _uselast) or (_uselast and (rows(_lnzofs) /= rows(_sg)));

            _lnzofs = ((1-_gam).*(_sg-_sbar));
            _lnzofs = _lnzofs + ln(exp((1-_gam).*_sbar)./
                (1-_delt*_bigg^(1-_gam).*_phi.*exp((1-_gam)^2*(1+1/_phi)^2
                            *_sigv^2/2)));

                     /* see notes for derivation of this approximation */

            /*
            _lnzofs = zeros(rows(_sg),1); /* start at zero option */
            */

            newlnz = _lnzofs;
            lnzhist = _lnzofs;

            iter = 1;                       /* loop to find z(s) */
            do while iter <= 100*_tsc;       /* didn't bother to set up test. */
                                            /* converges great in 200 iter.  */
              i = 1;
              do while i <= rows(_sg);
                _s = _sg[i];
                newlnz[i] = ln(exp(_s)^(1-_gam) +
                         intquad1(&zint,_sigv*_intlim));   /* update z */
                i = i+1;
              endo;
                               /* graph every 50 iter to see how you're doing */

              if _prniter and (abs(trunc(iter/50) - iter/50) < 0.001);
                  lnzhist = lnzhist~newlnz;
                  graphjc;
                  title("lnZ(s)");
                  px = _sg;
                  py = lnzhist;
                  _plctrl = zeros(cols(py)-1,1)|1;
                  _pstype = 4;
                  _ptek = "lnzofs.tkf";
                  xy(px,py);
              endif;

              _lnzofs = newlnz;
              iter = iter+1;
            endo;
            save _lnzofs;
        endif;       /* ends if not uselast */


        /* ------------------------------------------------------- */
        /* next step.                                             */
        /* maginal utility = C^(-gamma) * f(s). DO f(s) on a grid */
        /* ------------------------------------------------------- */

        _lnfmu = zeros(rows(_sg),1);
        i = 1;
        do while i <= rows(_sg);
            _s = _sg[i];
            _lnfmu[i] = intquad1(&mut1,_sigv*_intlim);
            i = i+1;
        endo;

        /*        another cool graph. uncomment to see it.
        graphjc;
        title("Log of Internal expectation term");
        _plctrl = 1; _psymsiz = 1; _pstype = 4;
        _ptek = "intexpec.tkf";
        xy(_sg,ln(_lnfmu));
        */

        /* see formula in paper. This is marginal utility */
        _lnfmu = ln(exp(_sg*(1-_gam)) .* (1+lamofs(_sg))
                       - lamofs(_sg).*_delt.*_bigg^(1-_gam).*_lnfmu);

        /* comparison graph. Now graphint.prg does a pretier version
        graphjc;
        fonts("simplex simgrma");
        title("Internal and external marginal utility");
        _plegstr = "Internal\000External";
        _plegctl = 1;
        _plctrl = 1; _psymsiz = 1; _pstype = 4;
        py = (-_gam)*_sg;
        py = py @+ _lnfmu[ctrindx]-py[ctrindx]@;
        xlabel("Log surplus consumption ratio s");
        ylabel("Log marginal utility");
        _ptek = "intmargu.tkf";
        xy(_sg,_lnfmu~py);
        */

    endif;

     /* --------------------------------------- */
     /* set up starting values for asset prices */
     /* --------------------------------------- */

     if _lnrf0 > 0;                                /* perpetuity price */
         _lnppa = -ln(exp(_lnrf0)-1)*ones(rows(_sg),1);
     else;
        _lnppa = zeros(rows(_sg),1);
     endif;

    _lnpda = 0*ones(rows(_sg),1);                 /* start others at zero*/
                                                  /* i.e. P/D = 1 */
    /* ----------------------- */
    /* get ready to loop       */
    /* initialize pd and newpd */
    /* ----------------------- */


    if _uselast;
        load _lnpd,_lnpdx,_lnpd1,_lnpcd,_lnpp,_lnpsc;
    endif;

    if (not _uselast) or (_uselast and (rows(_lnpd) /= rows(_sg)));
                          /*  ooops. last was a diff. grid */
       _lnpd = _lnpda;
       _lnpdx = ln(50)*ones(rows(_sg),1);
                    /* start at high value,wait till converges w start at 0 */
       _lnpd1 = _lnpda;
       _lnpcd = _lnpda;
       _lnpp = _lnppa;
       _lnpsc = _lnpda;
    endif;

    newlpd = _lnpd;                            /* initialize updated p/d */
    newlpdx = _lnpdx;
    newlpd1 = _lnpd;
    newlpcd = _lnpcd;
    newlpp = _lnppa;
    newlpsc = _lnpsc;

    /* --------------------------- */
    /* loop: update ln pd on grid  */
    /* --------------------------- */

    "Iterating P/C on grid. Have patience...." ;
    iter = 1;
    do while  (iter <= _maxit*_tsc)  ;

        if _prniter;
         "starting iteration " iter " of " _maxit*_tsc
               "p/d at sbar" _lnpd[ctrindx];
        endif;

        i = 1;                                /* loop thru gridpoints */
        do while i <= rows(_sg);

        /* ------------------------------------------------------------ */
        /* a reminder of what _assets means                             */
                        /* 1cons claim  2div 3d-c 4perp. 5bonds 6c*S    */
        /*    _assets =       1       |  0  |  0 |  0  |  0   | 1 ;     */
        /* ------------------------------------------------------------ */

            _s = _sg[i];      /* global value of today's state for sub procs */

                              /* integration subprocs are all below.        */
                              /* search on pdint to see them                */
                              /* _stock tells integration proc which integral*/
                             /* to take */
            if _assets[1];
                _stock = 0;
                newlpd[i] = ln(intquad1(&pdint,_sigv*_intlim));   /* update */
                _stock = 1;
                newlpdx[i] = ln(intquad1(&pdint,_sigv*_intlim));
            endif;
            if _assets[2];
                _stock = 2;
                newlpd1[i] = ln(intquad1(&pdint,_sigv*_intlim));
            endif;
            if _assets[3];
                _stock = 3;
                newlpcd[i] = ln(intquad1(&pdint,_sigv*_intlim));
            endif;
            if _assets[4];
                _stock = 4;
                newlpp[i] = ln(intquad1(&pdint,_sigv*_intlim));
            endif;
            if _assets[6];
                _stock = 6;
                newlpsc[i] = ln(intquad1(&pdint,_sigv*_intlim));
            endif;

            i = i+1;
        endo;

        /* graph prices to watch convrgence */

        if _prniter and (abs(trunc(iter/50) - iter/50) < 0.001);
            graphjc;
            px = _sg;
            py = newlpd~newlpdx~newlpd1~
                 newlpcd~newlpp~_lnpda~_lnppa~newlpsc;
            _plctrl = 1|1|1|1|1|0|0;
            _plegstr =
                 "p/c\000p/cx\000p/d1\000p/c(div)\000perp."
                  $+ "\000start: pda\000start: ppa\000S*C";
            _plegctl = 1;
            xy(px,py);
        endif;


        /* set up test values for convergence */

        {testval,testind} = testit(_lnpd,_lnpdx);
                             /* so far only testing _lnpd */

        _lnpd = newlpd;           /* always use last value */
        _lnpdx = newlpdx;
        _lnpd1 = newlpd1;
        _lnpcd = newlpcd;
        _lnpp = newlpp;
        _lnpsc =newlpsc;

        format /RZ 10,6;

        if maxc(abs(testval)) < _convtol;
            "p/d converged in " iter " iterations . last % diff" 100*testval';
            break;
        endif;

        if _prniter;
           "  max % this iter" 100*testval';
           "          at indx" testind' "of" rows(_sg) ;
        endif;

        iter = iter+1;
     endo;


    if iter >= _maxit*_tsc;
        "P/C loop did not converge in " iter " iterations";
        "last max % ch. " 100*testval  " at index" testind "of" rows(_sg);
    endif;

    save _lnpd,_lnpdx,_lnpd1,_lnpcd,_lnpp,_lnpsc;

    retp(_sg,_lnpda,_lnpd,_lnppa,_lnpp,_lnpdx,_lnpd1,
        _lnpcd,ctrindx,_lnpsc);

endp;

/* -------------------------------------------------------- */
/*               proc finders;

finds expected returns and other functions of state
typically by numerical integration e.g. for expected returns
Many subprocs for integrands are at the end of the proc.

Usage:
    {er,erd,erc,elnr,elnrdb,sdr,sdrd,sdrc,sdlnr,sdlnrdb,
              lnrf,lny,lny2b,slpmv,shpint} =  finders(_sg);
Note: uses lots of _globals, including _sg.


*/
/* ----------------------------------------------------------- */

clearg _mat;  /* current value of maturity, passed to integration proc */


proc(16) = finders(_zg);
    local er,erd,erc,elnr,elnrp,
    sdr,sdrd,sdrc,sdlnr,sdlnrp,lnrf,lnrf1,rf,i,
    lnp2,lnp2b,lny2b,lnyp,
    lnp1,lnp,j,lny,elnrdb,sdlnrdb,
    em1,sigmrs1,shpint,slpmv;

    /* -------------------------- */
    /* zero coupon term structure */
    /* -------------------------- */


    clear lnp1,lnrf,lnrf1,lnp2,lnp2b,lny2b,lny,lnyp,slpmv,shpint;

    lnp1 =  ln(_delt)-_gam*ln(_bigg) + _gam*(1-_phi)*(_sg-_sbar)
               +1/2*(_gam *_sigv * (lamofs(_sg)+1))^2 ;
    lnrf = -lnp1;

    if _b /= 0;

        /* ----------------------------------- */
        /* higher maturity pure discount bonds */
        /* ----------------------------------- */

        _lnpdb=lnp1;                  /* big matrix of all bond prices */
        lnp = zeros(rows(_sg),1);     /* holding area for next discount bond */
        if _maxdb > 1/_tsc;
            j = 2;
            do while j <= _maxdb*_tsc;
                i = 1;
                do while i <= rows(_sg);
                     _s = _sg[i];
                     lnp[i] = ln(intquad1(&intpdb,_sigv*_intlim));
                     i = i+1;
                endo;
                _lnpdb = _lnpdb~lnp;
                j = j+1;
            endo;
        endif;

       lny = -_lnpdb./(seqa(1/_tsc,1/_tsc,_maxdb*_tsc)');  /* yields */

  endif;     /* b/=0  */

    /* ------------------------------------------------------- */
    /* perpetuity yield: from P = sum Y^-j * 1 , Y = 1 + 1/P   */
    /* ------------------------------------------------------- */

    lnyp = ln(1+1./exp(_lnpp));

    /* ------------ */
    /* slope of mvf */
    /* ------------ */

    slpmv = (exp(_gam^2*_sigv^2*(1+lamofs(_sg))^2)-1)^(1/2);

    /* ---------------------------------------- */
    /* expected returns and standard deviations */
    /* ---------------------------------------- */

    lnrf1 = zeros(rows(_sg),1);          /* risk free to check and compare */
    er = zeros(rows(_sg),1);             /* cons claim return */
    erd = zeros(rows(_sg),1);            /* div growth stock return */
    erc = zeros(rows(_sg),1);             /* d-c stock return */
    elnr = zeros(rows(_sg),1);           /* log cons claim return */
    elnrdb = zeros(rows(_sg),_maxdb*_tsc);    /* zero coupon bonds */

    sdr = zeros(rows(_sg),1);
    sdrd = zeros(rows(_sg),1);
    sdrc = zeros(rows(_sg),1);
    sdlnr = zeros(rows(_sg),1);
    sdlnrdb = zeros(rows(_sg),_maxdb*_tsc);

    sigmrs1 = zeros(rows(_sg),1);
    shpint = zeros(rows(_sg),1);

    i = 1;
    do while i <= rows(_sg);
        _s = _sg[i];

        if _intrnl;
            em1 = intquad1(&intemrs1,_sigv*_intlim);
            lnrf1[i] = -ln(em1);
            sigmrs1[i] = intquad1(&intm2in,_sigv*_intlim);
            sigmrs1[i] = (sigmrs1[i]-em1^2)^0.5;
            shpint[i] = sigmrs1[i]/em1;
        else;
            lnrf1[i] = -ln(intquad1(&intemrs,_sigv*_intlim));
        endif;
        if _assets[1];   er[i] = intquad1(&inter,_sigv*_intlim);   endif;
        if _assets[2];  erd[i] = intquad1(&interd,_sigv*_intlim); endif;
        if _assets[3];  erc[i] = intquad1(&interc,_sigv*_intlim);  endif;
        if _assets[1]; elnr[i] = intquad1(&intelnr,_sigv*_intlim);endif;

        if _assets[1];
            sdr[i] = intquad1(&inter2,_sigv*_intlim);
            sdr[i] = (sdr[i]-er[i]^2)^(1/2);
        endif;
        if _assets[2];
            sdrd[i] = intquad1(&inter2d,_sigv*_intlim);
            sdrd[i] = (sdrd[i]-erd[i]^2)^(1/2);
        endif;
        if _assets[3];
            sdrc[i] = intquad1(&inter2c,_sigv*_intlim);
            sdrc[i] = (sdrc[i]-erc[i]^2)^(1/2);
        endif;
        if _assets[1];
            sdlnr[i] = intquad1(&intelnr2,_sigv*_intlim);
            sdlnr[i] = (sdlnr[i]-elnr[i]^2)^(1/2);
        endif;

       if (not (_b == 0));
            _mat = _maxdb*_tsc;
            do while _mat >= 2;
                elnrdb[i,_mat] = intquad1(&intelrdb,_sigv*_intlim);
                sdlnrdb[i,_mat] = intquad1(&intelrt2,_sigv*_intlim);
                sdlnrdb[i,_mat] = (sdlnrdb[i,_mat]-elnrdb[i,_mat]^2)^(1/2);
                _mat = _mat -1;
            endo;
            elnrdb[i,1] = lnrf[i];
        endif;

        i = i+1;
    endo;

    save er,erd,erc,elnr,elnrdb,sdr,
             sdrd,sdrc,sdlnr,sdlnrdb,
              lnrf,lnrf1,lny,lny2b,slpmv,shpint;

    retp(er,erd,erc,elnr,elnrdb,
         sdr,sdrd,sdrc,sdlnr,sdlnrdb,
              lnrf,lnrf1,lny,lny2b,slpmv,shpint);
endp;


/* ------------------------------------------ */
/* procs for numerical integration used above */
/* ------------------------------------------ */

proc(1) = norm(v);
    retp( 1/((2*pi)^(1/2).*_sigv) .* exp(-(v^2)/(2.*_sigv^2)) );
endp;

/* ------------------ */
/* internal mu Z term */
/* ------------------ */

proc(1) = zinsd(v);
    retp( _delt*_bigg^(1-_gam)*_phi*
          exp((1-_gam)*v).*(1+lamprim(_s).*v).*
          exp(interp(strans(_s,v),_sg,_lnzofs))
         );
endp;
proc(1) = zint(v);
    retp(norm(v) .* zinsd(v));
endp;

/* -------------------------- */
/* internal marg. util from z */
/* gives expectation term of  */
/*   marginal utility formula */
/* -------------------------- */

proc(1) = mut1ins(v);
    retp( exp((1-_gam)*v) .*
          exp(interp(strans(_s,v),_sg,_lnzofs)) .*
          (1 - _phi -_phi.*v.*lamprim(_s)) );
endp;

proc(1) = mut1(v);
    retp(norm(v) .* mut1ins(v));
endp;

/* ------- */
/* returns */
/* ------- */

proc(1) = erinsd(v);
 retp(  ( 1 + exp(interp(strans(_s,v),_sg,_lnpd)))
                            ./exp(interp(_s,_sg,_lnpd)) .* _bigg .* exp(v));
endp;

proc(1) = inter(v);
    retp( norm(v) .* erinsd(v) );
endp;

proc(1) = intelnr(v);
    retp( norm(v) .* ln(erinsd(v)) );
endp;

/*  ---------------------------------------------------------------------
    for dividend growth stock

           (1+P/D')  D'                w              rho sigw/sigv v + ...
   Er =  E[ ------- ---  ] = E [ .. G e  ] =  E[.. G e                      ]
             P/D     D

   like P/D recursion
------------------------------------------------------------------------ */


proc(1) = erdinsd(v);
 retp(  ( 1 + exp(interp(strans(_s,v),_sg,_lnpd1)))
                            ./exp(interp(_s,_sg,_lnpd1)) .* _bigg .*
                exp(_rhow.*_sigw./_sigv.*v).*exp(1/2*(1-_rhow^2)*_sigw^2)  );
endp;

proc(1) = interd(v);
    retp( norm(v) .* erdinsd(v) );
endp;

/* -------------------------------------------------------------
 for d-c stock

                            C'/C        P'     w'      v
    r = ( P'/C' + D'/C' ) * ------  = (---  + e  ) * Ge   ---
                            P/C         C'                 P/C

                       1/2sigw^2    v
   E(r) = E[ (P'/C' + e         ) Ge /(P/C)  ]

----------------------------------------------------------------- */


proc(1) = ercinsd(v);
 retp(  ( exp(interp(strans(_s,v),_sg,_lnpcd)) + exp(1/2*_sigwc^2 ))
                    ./exp(interp(_s,_sg,_lnpcd)) .* _bigg .* exp(v));
endp;

proc(1) = interc(v);
    retp( norm(v) .* ercinsd(v) );
endp;


/* --------------------------------- */
/* return^2 for conditional variance */
/* --------------------------------- */

proc(1) = inter2(v);
    retp( norm(v) .* erinsd(v)^2 );
endp;

proc(1) = intelnr2(v);
    retp( norm(v) .* ln(erinsd(v))^2 );
endp;


/* --------------------------------------------------
 for dividend growth stock


            (1+P/D') 2  D' 2            2 2w
  Er^2 = E{[ ------ ]  ---  ] = E [ .. G e   ] =
              P/D       D

          2E(w|v)  + 1/2 * (2^2 * sigma^2(w|v))
  E[  .. e                                       ]  =

          2E(w|v)  + 2 * sigma^2(w|v))
  E[  .. e                                ]  =

-------------------------------------------------- */

proc(1) = erd2ins(v);
 retp((  ( 1 + exp(interp(strans(_s,v),_sg,_lnpd1)))
                            ./exp(interp(_s,_sg,_lnpd1)) .* _bigg .*
                exp(_rhow.*_sigw./_sigv.*v).*exp((1-_rhow^2)*_sigw^2) )^2);
endp;

proc(1) = inter2d(v);
    retp( norm(v) .* erd2ins(v) );
endp;

/* -------------------------------------------------------------
    for d-c stock

     2       P'     w' 2   2 2v    -2
    r    = (---  + e  ) * G e   P/C     =
             C'
                  2             1/2sw^2     2sw^2     2  2v    -2
            (P'/C'   + 2 P'/C' e         + e       ) G  e   P/C


can't just square ercinsd
----------------------------------------------------------------- */

proc(1) = ercins2(v);
    local pc1;
    pc1 = exp(interp(strans(_s,v),_sg,_lnpcd));
    retp( ( pc1^2 + 2*exp(1/2*_sigwc^2 )*pc1 + exp(2*_sigwc^2)  )
                    ./exp(interp(_s,_sg,_lnpcd))^(2) .* _bigg^(2) .* exp(2*v));
endp;

proc(1) = inter2c(v);
    retp( norm(v) .* ercins2(v) );
endp;


/* --- */
/* mrs */
/* --- */

proc(1) = mrsinsd(v);
    retp( _delt .* _bigg^(-_gam) .* exp(-_gam*v)
            .* exp( -_gam * (strans(_s,v)-_s)));
endp;

proc(1) = intemrs(v);
    retp( norm(v) .* mrsinsd(v) );
endp;

proc(1) = mrs1insd(v);
    retp(_delt .* _bigg^(-_gam) .* exp(-_gam*v)
            .*  exp( interp(strans(_s,v),_sg,_lnfmu)-interp(_s,_sg,_lnfmu) ));
endp;

proc(1) = intemrs1(v);
    retp(norm(v) .* mrs1insd(v));
endp;

proc(1) = intm2in(v);
    retp(norm(v) .* mrs1insd(v)^2);
endp;

/* -------------- */
/* term structure */
/* -------------- */

/* ----------------------------------------------------------- */
/* general purpose term structure proc for higher order bonds. */
/* pays interpolated numerical last col of _lnpdb              */
/* ----------------------------------------------------------- */

proc(1) = intpdb(v);
    retp(norm(v).* mrsinsd(v).*
            exp(interp(strans(_s,v),_sg,_lnpdb[.,cols(_lnpdb)])));
endp;

/* ----------------------------------- */
/* term structure expected return proc */
/* ----------------------------------- */

proc(1) = erdbin(v);     /* P(one lower mat tomorrow) / P (this mat now ) */
     retp((exp(interp(strans(_s,v),_sg,_lnpdb[.,_mat-1])))
                    ./exp(interp(_s,_sg,_lnpdb[.,_mat])) );
endp;

proc(1) = interdb(v);
    retp( norm(v) .* erdbin(v) );
endp;

proc(1) = intelrdb(v);
    retp( norm(v) .* ln(erdbin(v)) );
endp;

proc(1) = intelrt2(v);
    retp( norm(v) .* ln(erdbin(v))^2 );
endp;



/* -------------------------------------------------------------------------
                     proc simvars
       simulates time series of variables.

Usage:
        call simvars(dctb,wtb,wctb);

creates the following global variables in workspace

 stb,lndctb,lnpdtb,lnpd1tb,lnpdctb,
         lnrtb,lnrdtb,lnrctb,
         ertb,erdtb,erctb,elnrtb,
         sdrtb,sdrdtb,sdrctb,sdlnrtb,
         lnrftb,elnrdbtb,lnytb,lnrdbtb,
         wtb,wctb;


convention: xtb = time series of t at simulation frequency.
Derives all time series
Uses global variables to look up functions of state, _lnpd etc. and er etc.

creates globals in workspace, since it was running out of memory trying to
return copies of locals.

if you supply consumption or dividend growth it will use it. If
you supply 0, it simulates it.

wt = ln dividend growth
wct = ln d - ln c (mean 0);

level series (functions of state st )
are initialized, so start one earlier and are one larger
than growth series. 1 = long, 0 = short.

{st,lndct,lnpdt,lnpd1t,lnpdct,
  1    0    1     1       1
lnrt,lnrdt,lnrct,ert,erdt,erct,elnrt
  0    0     0    1   1     1    1
 sdrt,sdrdt,sdrct,sdlnrt,
   1     1    1    1
lnrft,elnrdbt,lnyt,lnrdbt,wt,wct} = simvars(0,0,0);
   1    1     1    1      0   1


-------------------------------------------------------------------------- */

proc(0) = simvars(arg1,arg2,arg3);
    local dctb,vtb,T,i,lnpcdtb,lny2tb;
     dctb = arg1;
     wtb = arg2;
     wctb = arg3;


    "Simulating artificial time series...";


     /* ------------------------------ */
     /* simulate consumption,dividends */
     /* ------------------------------ */

    if dctb == 0;
        T = _ncalc;
        vtb = _sigv*rndn(T,1);                      /* shock series */
        lndctb = ln(_bigg) + vtb;                     /* cons. growth */
    else;
        T = rows(dctb);
        if minc(dctb) <= 0;
            "simvars: you gave me a consumption growth < 0. I need "
            " gross consumption growth, not log or net.";
        endif;
        lndctb = ln(dctb);
        vtb = lndctb - ln(_bigg);
    endif;

    if _assets[2];
        if wtb == 0;
            wtb = _rhow*_sigw/_sigv*vtb + _sigw*(1-_rhow^2)^0.5*rndn(T,1);
		lndtbcc=ln(_bigg)+wtb;
        else;
            if rows(wtb) ./= rows(lndctb);
                "simvars: you passed me incompatible consumption and dividends";
            endif;
        endif;
    endif;

    if _assets[3];
        if wctb == 0;
            wctb = _sigwc*rndn(T+1,1);
        else;
            if rows(wctb) ./= rows(lndctb);
                "simvars: you passed me incompatible consumption and dividends";
            endif;
        endif;
    endif;

    /* simulate s */

    stb = zeros(T+1,1);                            /* s */
    if _s0 == 0;
        _s0 = _sbar;                             /* place to start */
    endif;

    stb[1] = _sbar;                              /* initialize s */

    i = 2;                                      /* simulate s process */
    do until i > T+1;
       stb[i] = strans(stb[i-1],vtb[i-1]);         /* v is one shorter than s */
       i=i+1;
    endo;
    _s0 = stb[rows(stb)];         /* start next one here */

    /* ------------------------- */
    /* derive other time series  */
    /* ------------------------- */

    /* ------------- */
    /* 1. p/d ratios */
    /* ------------- */

    if _assets[1];
        lnpdtb = interp(stb,_sg,_lnpd) ;          /* price/consumption ratio */
    endif;
    if _assets[2];
        lnpd1tb = interp(stb,_sg,_lnpd1) ;         /* p/d of div growth model*/
	  lnptcc=lndtbcc+lnpd1tb[2:rows(lnpd1tb)]-lnpd1tb[1:rows(lnpd1tb)-1];
	  i=2;do while i<T+2;
	  _divid[i]=_divid[i-1]+lndtbcc[i-1];
	  _profstock=_divid+lnpd1tb;i=i+1;endo;
    endif;

    if _assets[3];
        lnpcdtb = interp(stb,_sg,_lnpcd) ;        /* p/c of d-c model*/
    endif;

    /* ------------------ */
    /* 2. ex-post returns */
    /* ------------------ */

    /* -------------------- */
    /* a. consumption claim */
    /* -------------------- */
                /* R = (1+P'/D') / P/D * D'/D */

    lnrtb = ln( 1+exp(lnpdtb[2:T+1]) ) - lnpdtb[1:T] + lndctb ;

    /* ------------------------ */
    /* b. dividend growth model */
    /* ------------------------ */

    if _assets[2];
        lnrdtb =ln( 1+exp(lnpd1tb[2:T+1]) ) - lnpd1tb[1:T] + ln(_bigg) + wtb;
    endif;

    /* ---------------------------------------- */
    /* c. d-c model                             */
    /* ---------------------------------------- */

    if _assets[3];
        lnpdctb = lnpcdtb - wctb;   /* P/D = P/C * C/D */

                                    /* R = (D'/C'+ P'/C') * C/P *C'/C */
        lnrctb = ln(exp(wctb[2:T+1])+exp(lnpcdtb[2:T+1]))
                         - lnpcdtb[1:T] + lndctb;
        clear lnpcdtb;
    endif;

    /* -------------------------------------- */
    /* d. rf for varying rf case and as check */
    /* -------------------------------------- */

    /*    ignore region above s max by commenting this out.
        lnrft = -ln(_delt)+_gam*ln(_bigg) - _gam*(1-_phi)*(st-_sbar)
                 -1/2*(_gam *_sigv * (lamofs(st)+1))^2 ;
    */
    lnrftb = _lnrf0*ones(rows(stb),1);

     /* --------------------------------------- */
     /* 3. Expected returns and conditional s.d.*/
     /* --------------------------------------- */

    if _condtim;
        if _assets[1]; elnrtb  = interp(stb,_sg,elnr); endif;
        if _assets[1]; sdlnrtb = interp(stb,_sg,sdlnr); endif;
        if _assets[1]; ertb    = interp(stb,_sg,er); endif;
        if _assets[2]; erdtb   = interp(stb,_sg,erd); endif;
        if _assets[3]; erctb   = interp(stb,_sg,erc); endif;
        if _assets[1]; sdrtb   = interp(stb,_sg,sdr); endif;
        if _assets[2]; sdrdtb  = interp(stb,_sg,sdrd); endif;
        if _assets[3]; sdrctb  = interp(stb,_sg,sdrc); endif;
    endif;

    if not (_b == 0);
       elnrdbtb = interp(stb,_sg,elnrdb[.,1]);      /* expected bond returns */
       lnytb    = interp(stb,_sg,lny[.,1]);         /* bond yields           */
       lny2tb =   zeros(rows(lnytb),1);          /* yield of 1 less mat. bond*/
        i = 2;
        do while i <= cols(lny);
            if maxc( i .== bondsel*_tsc) ;
                lnytb = lnytb~interp(stb,_sg,lny[.,i]);
                lny2tb = lny2tb~interp(stb,_sg,lny[.,i-1]);
                elnrdbtb = elnrdbtb~interp(stb,_sg,elnrdb[.,i]);
            endif;
            i = i+1;
        endo;

        lnrdbtb = 0|lnytb[1:T-1,1];
        i = 2;
        do while i <= rows(bondsel);
            lnrdbtb = lnrdbtb~
                     (0|(-lny2tb[2:T,i]*((bondsel[i]-1/_tsc))+
                                  lnytb[1:T-1,i]*bondsel[i])/_tsc);
            i = i+1;
        endo;
   endif;

endp;


/* -------------------------------------------------------------------------- */
/*
    proc annvars makes annual observations of high frequency simulated
    series for comparison to annual observations in data

     note if you want to pass dc, etc., these are passed right
    to simvars at high frequency.

    also returns high frequency simulated series with a b suffix.
    from internal call to simvars

   retunrs globals

         lndct,lnpdt,lnpd1t,lnpdct,lnrt,lnrdt,lnrct,
         lnrft,lnyt,lnrdbt,lnddt,lnchpt,lnchpdt,st,
         stb,lndctb,lnpdtb,lnpd1tb,lnpdctb,lnrtb,lnrdtb,lnrctb,ertb,
         erdtb,erctb,elnrtb,
         sdrtb,sdrdtb,sdrctb,sdlnrtb,lnrftb,elnrdbtb,lnytb,lnrdbtb,wtb,wctb
   returned as globals so you won't run out of workspace memory trying to
send back copies of long series.

syntax:

     Call  annvars(dct,wt,wct);


    note: timing as in data; r,dp are the same length as other series
*/
/* -------------------------------------------------------------------------- */

proc(0) = annvars(arg1,arg2,arg3);
    local T,lnct,lntb,lndt,lnchptb;

    /* form high-frequency versions of simulated data */

     call simvars(arg1,arg2,arg3);

     T = rows(stb);

     /* now convert to annual time series */

     /* sum level of consumption,dividends to make annual */

     lnct = cumsumc(lndctb);     /* makes monthly log level of consumption*/
     lnct = ln(chgfreq(exp(lnct),_tsc,_tsc,0));   /* sums over 12 months */
     lndct = lnct[2:rows(lnct)]-lnct[1:rows(lnct)-1];   /* annual growth */
     clear lnct;

     if _assets[2];
         lnddtb = wtb+ln(_bigg);
         lndtb = cumsumc(lnddtb);
         lndt = ln(chgfreq(exp(lndtb),_tsc,_tsc,0));
         lnddt = lndt[2:rows(lndt)]-lndt[1:rows(lndt)-1];
         clear lndt,lntb;
    endif;

     /* report end-of-year p/d ratio scaled to annual dividends */
     /* and report end-of-year spc to construct a discount factor */

     if rows(lnpdtb) > 1;
         lnpdt = chgfreq(lnpdtb[2:T],1,_tsc,0)-ln(_tsc);
         lnpdt = lnpdt[2:rows(lnpdt)];        /* 2: to match c growth */
     endif;
     if rows(lnpd1tb) > 1;
         lnpd1t = chgfreq(lnpd1tb[2:T],1,_tsc,0)-ln(_tsc);
         lnpd1t = lnpd1t[2:rows(lnpd1t)];
     endif;
     if rows(lnpdctb) > 1;
        lnpdct = chgfreq(lnpdctb[2:T],1,_tsc,0)-ln(_tsc);  /* is -_tsc right? */
        lnpdct = lnpdct[2:rows(lnpdct)];
     endif;
     if rows(stb) > 1;
        st = chgfreq(stb[2:T],1,_tsc,0);
        st = st[2:rows(st)];
     endif;

     /* product up returns to make annual returns */

     if rows(lnrtb) > 1;
        lnrt = chgfreq(lnrtb,_tsc,_tsc,0);
        lnrt = lnrt[2:rows(lnrt)];          /* 2 to match cons growth */
     endif;
     if rows(lnrdtb) > 1;
        lnrdt = chgfreq(lnrdtb,_tsc,_tsc,0);
        lnrdt = lnrdt[2:rows(lnrdt)];
     endif;
     if rows(lnrctb) > 1;
        lnrct = chgfreq(lnrctb,_tsc,_tsc,0);
        lnrct = lnrct[2:rows(lnrct)];
     endif;
     if rows(lnrftb) > 1;
        lnrft = chgfreq(lnrftb[1:T-1],_tsc,_tsc,0);  /* -1 so return dating */
        lnrft = lnrft[2:rows(lnrft)];
     endif;

     /*  change in price */

     if rows(lnpdtb) > 1;
         lnchptb  = (lnpdtb[2:T]-lnpdtb[1:T-1]+lndctb);
         lnchpt = chgfreq(lnchptb,_tsc,_tsc,0);
         lnchpt = lnchpt[2:rows(lnchpt)];
         clear lnchptb;
     endif;
     if rows(lnpd1t) > 1;
         lnchpdtb = (lnpd1tb[2:T]-lnpd1tb[1:T-1]+lnddtb);
         lnchpdt = chgfreq(lnchpdtb,_tsc,_tsc,0);
         lnchpdt = lnchpdt[2:rows(lnchpdt)];
     endif;
endp;



/* ---------------------------------------------- */
/* proc seqb
   Like seqa but specify min, max, and step size  */
/* ---------------------------------------------- */

proc seqb(min,max,step);
    local nsteps, x;
     if max < min ; "seqb: asking for max < min"; endif;
    nsteps = trunc((max-min)/step+1.000000000001);
    x = seqa(min,step,nsteps);
    retp(x);
endp;

/* ----------------------------------------------------------- */
/*                   proc strans
   describes transition equation for state variable s.

   news = strans(s,v)

   s = state at t (may be vertical vector)
   v = random normal (0,_sigv) shock (must be one dimensional)

This example implements

      s_{t+1}  =  (1-phi) _sbar  + phi s{t} + lamofs(s) *v

      _limits option keeps s' below smax.
--------------------------------------------------------------- */
proc strans(s,v);
  local news ;
  news =  (1-_phi)*_sbar + _phi*s + lamofs(s).*v;
  if _limits;
    news = (news .<= _smax).*news + (news .> _smax).*_smax;
  endif;
  retp( news);
endp;


/* ------------------------------------------------------------------
                             proc interp

general purpose linear interpolator.

    f(z) = interp(z,xgrid,fxgrid)

    z = vector of values at  which to find f(z)
    xgrid   = vector, grid point of values on which fx is defined
    fxgrid  = vector, current values of f(x) on that grid.
    f(z) = conformable vector of interpolated f(z)

    xgrid must be organized from low to high values
---------------------------------------------------------------------  */


/*
   finds a local slope and intercept to be used for each range in zgrid,
   then does f(x) = a+bx */

proc(1) = interp(zv,x,fx);
    local T,gradf,intcpt,slope,aindc,fofz,zl,transans,i;

    if fx == 0;       /* covers case that fx is not yet defined */
        retp(0);
    endif;

    zl = zv;

    T = rows(x);

    if x[2] < x[1]; "interp: grid must rise"; endif;
    transans=0;
    if cols(zv) > 1 and rows(zv) == 1;
        zv = zv';
        transans = 1;
    endif;
    if cols(zv) > 1 and rows(zv) > 1;
         "interp: Error: asking for matrix";
    endif;

    /* make vector of intercepts and slope coefficients to be used for range of
      z values.  vectors are arranged as follows

    xgrid and pd:          z1   z2   z3   ...  zT
    slopes and incpts   a1    a2  a3      ...aT   aT+1

    note it is longer than z grid!
    */

    gradf = (fx[2:T]-fx[1:T-1])./(x[2:T]-x[1:T-1]);
    intcpt = ( fx[1] - gradf[1]*x[1] )|
             (fx[1:T-1] - gradf.*x[1:T-1]) |
             (fx[T] - gradf[T-1]*x[T]) ;
    slope =  gradf[1]|(gradf)|gradf[T-1];


    if rows(zv) <= 1000;
        aindc = (zv .> x[1]).*(minindc(missex(zv'-x, zv'-x .< 0))+1) +
            (zv .<= x[1]);
                           /* what index of slope and intcpt vectors to use */
                           /* use next higher one  */
                           /* minindc finds index of next lowest x
                               minimum (positive) difference between zv and x */
                           /* missex tells it to ignore values less than zero*/
    endif;

    if rows(zv) > 1000;     /* other version is faster, but runs out of memory*/
        aindc = ones(rows(zv),1);
        i = 1;
        do while i <= rows(zv);
            if zv[i] > x[1];
                aindc[i] = minindc(missex(zv[i]-x, zv[i]-x .< 0))+1 ;
            endif;
            i = i+1;
        endo;
    endif;

    fofz = intcpt[aindc] + slope[aindc].*zv;

    if transans;
        fofz = fofz';
    endif;

    retp(fofz);
endp;

/* --------------------------------------------------------------
                          proc pdint,ppint
   used by intquad1 to integrate. Must have ONLY v~N(0,_sigv)
   as argument. Pass other things (today's state, parameters) as globals.
   Returns LEVEL of p/d,pp at current value of _s from one iteration
------------------------------------------------------------------ */

proc(1) = pdtomor(v);
    local inside, s1;
    s1 = strans(_s,v);
    if not _intrnl;

      if _stock == 0;                       /* cons. claim, recusision from 0 */
          inside = _delt .* _bigg^(1-_gam) .* exp(-_gam*(s1-_s)) .*
               (1 + exp(interp(s1,_sg,_lnpd))) .*  exp((1-_gam).*v) ;

      elseif _stock == 1;                   /* cons claim, recursion from ppa */
          inside = _delt .* _bigg^(1-_gam) .* exp(-_gam*(s1-_s)) .*
               (1 + exp(interp(s1,_sg,_lnpdx))) .*  exp((1-_gam).*v) ;

      elseif _stock == 2;                   /* iid dividend growth case */
          inside = _delt .* _bigg^(1-_gam) .* exp(1/2*(1-_rhow^2)*_sigw^2) .*
               exp(-_gam*(s1-_s)) .* (1 + exp(interp(s1,_sg,_lnpd1))) .*
                exp((_rhow*_sigw/_sigv -_gam).*v) ;

      elseif _stock == 3 ;                  /* iid d/c ratio case */
          inside = _delt .* _bigg^(1-_gam) .* exp(-_gam*(s1-_s)) .*
           (exp(1/2*_sigwc^2) + exp(interp(s1,_sg,_lnpcd))).* exp((1-_gam).*v) ;

      elseif _stock == 4;                  /* perpetuity */
          inside = _delt .* _bigg^(-_gam) .* exp(-_gam*(s1-_s)) .*
               (1 + exp(interp(s1,_sg,_lnpp))) .*  exp(-_gam.*v) ;

      elseif _stock == 6;                  /* S*C for risk aversion */
          inside = _delt .* _bigg^(1-_gam) .* exp(-_gam*(s1-_s)) .*
           (exp(s1) + exp(interp(s1,_sg,_lnpsc))) .* exp((1-_gam).*v) ;

               /* P0 = E0 [ M1 (S1C1 + P1 )]*/
               /* P0/C0 = E0[ M1 (S1 C1/C0 + P1/C1 C1/C0)]  */
               /* P0/C0 = E0[ M1 C1/C0 (S1 + P1/C1)]  */
      endif;
    endif;

    if _intrnl;
      if _stock == 0;                       /* cons. claim, recusision from 0 */
          inside = _bigg .* mrs1insd(v) .*
                       (1 + exp(interp(s1,_sg,_lnpd))) .*  exp(v) ;

      elseif _stock == 1;                   /* cons claim, recursion from ppa */
          inside = _bigg .* mrs1insd(v) .*
                       (1 + exp(interp(s1,_sg,_lnpdx))) .*  exp(v) ;

      elseif _stock == 2;                   /* iid dividend growth case */
          inside = _bigg .* exp(1/2*(1-_rhow^2)*_sigw^2) .*  mrs1insd(v) .*
                       (1 + exp(interp(s1,_sg,_lnpd1))) .*
                                                 exp(_rhow*_sigw/_sigv .*v) ;
      elseif _stock == 3 ;                  /* iid d/c ratio case */
          inside = _bigg .* mrs1insd(v) .*
           (exp(1/2*_sigwc^2) + exp(interp(s1,_sg,_lnpcd))).* exp(v) ;

      elseif _stock == 4;                  /* perpetuity */
          inside =  mrs1insd(v) .* (1 + exp(interp(s1,_sg,_lnpp))) ;

      elseif _stock == 6;                  /* S*C for risk aversion */
          "Warning: S*C calculaiton for risk aversion may not work internal";
          inside = mrs1insd(v) .*  _bigg .* exp(v) .*
           (exp(s1) + exp(interp(s1,_sg,_lnpsc)))  ;

      endif;


    endif;
    retp(inside);
endp;


proc(1) = pdint(v);
    local inside;
    inside = 1/(_sigv * (2*pi)^(1/2) ) .*  exp(-(1/2).*(v./_sigv)^2) .*
             pdtomor(v);
    retp(inside);
endp;



/* ----------------------------------------------------   */
/*                      FILL.G                            */
/*  proc fill to put series of different lengths on graph */
/* ----------------------------------------------------   */

proc(1) = fill(x,y);
    if rows(x) < rows(y);
        retp( (x|(ones(rows(y)-rows(x),1).*x[rows(x),.]))~y);
    endif;
    if rows(x) > rows(y);
        retp( x~(y|(ones(rows(x)-rows(y),1).*y[rows(y),.])));
    endif;
    if rows(x) == rows(y);
        retp(x~y);
    endif;
endp;

/* -----------------------
proc cov takes covariances
EITHER a OR b can have multiple columns
-------------------------- */

proc(1) = cov(a,b);
    if rows(a) /= rows(b); "proc cov: matrices must have same rows"; endif;
    retp( meanc( (a-meanc(a)') .* (b-meanc(b)') )');
endp;

/* ----------------------------------------- */
/* procs trans and back                      */
/* transform parameters to keep search       */
/* in (0,1) or (0,inf)                       */
/* not currently used                        */
/* ----------------------------------------- */

proc(1) = trans01(x);
    retp( (x-1/2)./(x.*(1-x))  );
endp;
proc(1) = back01(t);
    local ans;
    ans = ( t-1 + ( 1+t^2 )^(1/2) )./(2*t);
    if (ans < 0) or (ans > 1);
        ans = ( t-1 - ( 1+t^2 )^(1/2) )./(2*t);
        if (ans < 0) or (ans > 1);
            "back: cant invert ";
        endif;
    endif;
    retp(ans);
endp;
proc(1) = t0inf(x);
    retp(ln(x));
endp;
proc(1) = b0inf(t);
    retp(exp(t));
endp;



/* -------------------------------------------------------------------------- */
/*                               CHGFREQ                                      */
/*
     USAGE:

        chgfreq(returns, horizon , freqency, offset);

     1) offset = 0, frequency = 1:

     Takes monthly end-of month returns, in units log(R) so they may be added.
     returns monthly observations of
     k month overlapping log returns, dated as of last date.
     e.g  if r(t) is the return to end of month t, the program takes

     r(1)         0                            = ro(1)
     r(2)         0                            = ro(2)
     ..           ..
     r(k)   -->   r(1)+r(2)..+r(k)   = ro(k)
     r(k+1)       r(2)+r(3)..+r(k+1) = ro(k+1)
     ..           ..                              ..
     r(T)         r(T-k+1)+..+r(T)   = ro(T)

     NOTE returns SUMS not averages.

     2) frequency > 1, offset .ne. 0:

     samples every frequency points, starting at the frequency + offest'th.
     e.g. for freq = 3, o = 2,

     ro(1)   -->   rf(1)
     ro(2)
     ro(3)
     -----
     ro(4)   -->   rf(2)
     ro(5)
     ro(6)
       ..

    EXAMPLE:  from monthly data:
    to create quarterly data, with Q1 return = jan+feb+march,
    use horizon = 3; freqency  = 3, offset = 0;
    to create quarterly data, with Q1 return = nov+dec+jan,
    use horizon = 3; frequency = 3, offset = 2;
    to create quarterly observations of annual returns, with Q1 = feb+..+jan,
    use horizon = 12, frequency = 3, offset = 2;
    to create quarterly data, with Q1 data = march;
    use horizon = 1, frequency = 3, offset = 0;

    from quarterly data:
    to create annual data
    use horizon = 4, frequency = 4, offset = 0;
    to create quarterly observations of annual averages
    use horizon = 4, frequency = 1, offset = 0;
                                                                              */
/* -------------------------------------------------------------------------- */

proc(1) = chgfreq(rm,k,f,o);
    local bigr,i,T,mask,ro;
    T = rows(rm);
    ro = rm;

    if k > 1;
        bigr = rm[1:T-k+1,.];
        i = 1;
        do while i <= k-1 ;
            bigr = bigr+rm[1+i:T-k+1+i,.];
            i = i+1;
        endo;
        ro = ((-99*ones(k-1,1))*ones(1,cols(bigr)))|bigr;
    endif;

    if f > 1;
        mask = zeros(rows(ro),1);
        i = 1;
        do while i <= rows(ro);
            if f*i-o <= rows(ro);
                mask[f*i-o] = 1;
            else;
                break;
            endif;
            i = i+1;
        endo;
        ro = selif(ro,mask);
    endif;

    retp(ro);
endp;


/* -------------------------------------------------------------------------- */
/*                               CORR.G                                       */
/*
   Returns correlation between two series.

    Usage:

        a = corr(x,y)

    Note:

        x or y may be a matrices (1..T vertically).  In this case the correla-
    tion matrix between the series in x and y is returned.

    Note:
       The conventional standard error of a correlation is (1-rho^2)/root(n)

*/
/* -------------------------------------------------------------------------- */


Proc(1) = corr(x,y);
    local a;
    if rows(x) /= rows(y);
        "Corr: the two arguments to corr() must have the same no. of rows";
        retp(-999);
    endif;
    if rows(x) == 1;
        "Corr: the arguments to corr() must have more than one row";
        retp(-999);
    endif;
    a = (x-meanc(x)')'*(y-meanc(y)') ./ (stdc(x) * stdc(y)') / (rows(x)-1);
    retp(a);
endp;


/* -------------------------------------------------------------- */
/* graphjc                                                     */
/* sets up my own defaults for graphs, use in place of graphset */
/* -------------------------------------------------------------- */

proc(0) = graphjc;
graphset;
_pltype = 6;
_pdate = "";
_pframe = 0|0;
_pnum = 1|2;
endp;

/* ------------------------------------------------------------ */
/*                      loaddat.prg                           */
/* makes annual, quarterly price, return and consumption data

CHECK Timing assumptions on annual data with jyc.

There are three programs which assemble three datasetss.

1) qloaddat assembles quarterly postwar data
2) ploaddat assembles annual postwar data
3) aloaddat assembles campbell-shiller long data, merged a bit with #2.

the data are massaged somewhat so that they all obey the same timing
conventions etc.

Almost all data are the same length. T bill rate is return so does not have
to be shifted to create an excess return.
Exceptions:
1) GDIV is only 59-now,
2) campbell shiller consumption & deflator data are shorter than stock data.

General syntax:

{lnret,lnpd,lndd,lndelp,lnrf,lndc,lncpi,extras} = xloaddat;

lnret  = nominal stock return
lnpd   = p/d ratio. p/annual d.
lndd   = nominal dividend growth.
lndelp = nominal 
 growth.  (delp = deltap)
lnrf   = nominal risk free return proxy. Note return, thus r[1:T]-rf[1:T]
lndc   =  real consumption/capita
lndcpi = inflation rate cpi[t] - cpi[t-1], thus conformable with returns.

Syntax of individual procs:

{qlnret,qlnpd,qlndd,qlndelp,qlntb,qlndc,qlndcpi,qlnddns,qlngdiv} = qloaddat;
{alnret,alnpd,alndd,alndelp,alntb,alndc,alndcpi} = ploaddat(chopflg);
{lnspret,lnsppd,lndd,lndelp,lnir,lndc,lndppi,lndcdef,sdate,cdate} = aloaddat;

Notes:

qloaddat
Loads quarterly postwar data. all variables are 47:1-95:4.
pd is based on annual returns, e.g. dividends for the whole last year
reinvested at the market.
lvwdd is dividend growth based on that concept.thus is growth over the last year
lvwddns is raw quarterly div growth with a big seasonal!
lncpi is growth ratefrom sbbi.

aloaddat;
loads annual long time series. watch it: NOT all same length
--------------------------------------------------------------------------- */

/* --------------------------------------- */
/* proc ploaddat loads annual postwar data */
/* --------------------------------------- */

proc(7) = ploaddat(start);  /* start = 0: all pw. start = 1: stocks start 30*/
    local x,avwret,avwretx,vwdp,T,lvwdd,sbdate,lndc,
          agb,acb,aust,acpi,t1,t2,t3,t4,t5,t6,t7,t8,t9,qlnc,alnc,dalnc;

    /* -------------------------------------- */
    /* get stock return data from annual crsp */
    /* -------------------------------------- */

    load x[70,3] = \aaj\camrb\data\asia.dat;
    T = rows(x);
    avwret  = 1+x[.,2];              /* 26-95. 47 is 22 */
    avwretx = 1+x[.,3];
    vwdp = avwret./avwretx-1;
    lvwdd = ln(vwdp[2:T]) - ln(vwdp[1:T-1]) + ln(avwretx[2:T]);

    /* ------------------------- */
    /* get rf,cpi data from sbbi */
    /* ------------------------- */


     load x[70,5] = \aaj\camrb\data\asbbi.dat;
     sbdate = x[.,1];
     agb = 1+x[.,2];
     acb = 1+x[.,3];
     aust = 1+x[.,4];
     acpi = 1+x[.,5];

    /* ---------------------- */
    /* chop to postwar period */
    /* ---------------------- */

    if start == 0;
        avwret  = avwret[23:T];
        avwretx = avwretx[23:T];
        vwdp    = vwdp[23:T];
        lvwdd   = lvwdd[22:T-1];
        aust    = aust[23:T];
        acpi    = acpi[23:T];
    endif;

    if start == 1;
        avwret  = avwret[2:T];
        avwretx = avwretx[2:T];
        vwdp    = vwdp[2:T];
        lvwdd   = lvwdd[1:T-1];
        aust    = aust[2:T];
        acpi    = acpi[2:T];
    endif;

    /* ---------------------------------------------- */
    /* aggregate quarterly consumption, to get annual */
    /* ---------------------------------------------- */

    {t1,t2,t3,t4,t5,lndc,t7,t8,t9} = qloaddat;
    qlnc = cumsumc(0|lndc);
    alnc = ln(chgfreq(exp(qlnc),4,4,0));  /* add levels over time */
           "check for leading zeros in consumption calculation";
    dalnc = alnc[2:rows(alnc)]-alnc[1:rows(alnc)-1];

   retp(ln(avwret),-ln(vwdp),lvwdd,ln(avwretx),ln(aust),dalnc,ln(acpi));
endp;


/* ------------------------------------- */
/* qloaddat loads quarterly postwar data */
/* ------------------------------------- */

proc(9) = qloaddat;

    local  cn,cs,c,gpop,x,lnc,gb,tb,vwret,vwretx,T,ar,arx,vwdp,lvwpd,terms,
    yr1,q1,yr2,q2,cn2,cs2,c2,px,py,lvwdd,lvwddns,vwdpns,cpi,gdiv,punew,
    lndc,lndcpi;


    load x[189,4] = \aaj\camrb\data\cns.dat;
    yr1 = x[1:188,1];
    q1 = x[1:188,2];
    cn = x[1:188,3];       /* 47:1 to 94:1 */
    cs = x[1:188,4];
    c = cn + cs;

    load x[148,6] = \aaj\camrb\data\cns2.dat;
    yr2 = x[3:148,1];    /* first two obs are zero */
    q2 = x[3:148,2];
    cn2 = x[3:148,3];
    cs2 = x[3:148,5];
    c2 = cn2+cs2;

    c = c.*c2[2]/c[52]; /* make match in 59:4 */

/* if you want to see match, do this
    graphjc;
    px= fill(yr1+q1/4,yr2+q2/4);
    py = fill(c,c2);
    xy(px,py);
*/

    c = c[1:52]|c2[3:rows(c2)];   /* match in 59:4, then use new 60:1 to 95:4 */
                                  /* total is 47:1 to 95:4 */

    load x[200,3] = \aaj\camrb\data\gpop.dat;
    gpop =  x[.,3];           /* from 46:1 to 95:4 */

    /* nd&s consumption per capita */
    c = c./gpop[5:rows(gpop)];
    c = c/4;                              /* if it matters, pce is saar */
    lnc = ln(c);
    lndc = lnc[2:rows(lnc)]-lnc[1:rows(lnc)-1];

    /* nipa dividends per capita */

    load gdiv[148,3] = \aaj\camrb\data\gdiv.dat;
    gdiv = gdiv[.,3];
    gdiv = gdiv/4;
    gdiv = gdiv./gpop[1+rows(gpop)-rows(gdiv):rows(gpop)];

    /* ---------------------- */
    /* quarterly sbbi data    */
    /* make to 47:1-95:4      */
    /* ---------------------- */

    load x[280,5] = \aaj\camrb\data\qsbbi.dat;
    gb = 1+x[.,2];    /* data to 1995:4 */
    tb = 1+x[.,4];
    cpi = 1+x[.,5];
    gb = gb[86:rows(gb)]; /* 47:1-95:4 */
    tb = tb[86:rows(tb)];
    cpi = cpi[86:rows(cpi)];   /* use the better punew instead */

    load punew[591,3] = \aaj\camrb\data\punew.dat;
    punew = punew[1:588,3];           /* lop off 1996 observations */
    punew = chgfreq(punew,1,3,0);     /* qtrly by taking last month in qtr */
    lndcpi = ln(punew[2:rows(punew)]./punew[1:rows(punew)-1]);
                                      /* report growth in cpi */
/*
    graphjc;
    title("punew vs. sbbi inflation");
    xtics(45,95,5,0);
    _plegstr = "punew\000sbbi";
    _plegctl = 1;
    xy(seqa(45.25,0.25,rows(lndcpi)),lndcpi~ln(cpi));
*/
    /* make gdiv real with cpi */
    gdiv = gdiv./punew[rows(punew)-rows(gdiv)+1:rows(punew)];

    load x[280,3]=\aaj\camrb\data\qsia.dat;    /* 26:1 - 95:4*/
    vwret = 1+x[.,2];
    vwretx = 1+x[.,3];

    /* form quarterly annualized returns for d/p*/

    T = rows( vwret);
    ar  =  vwret[4:T].* vwret[3:T-1].* vwret[2:T-2].* vwret[1:T-3];
    arx =  vwretx[4:T].* vwretx[3:T-1].* vwretx[2:T-2].* vwretx[1:T-3];
    vwdp = ar./arx-1;    /* note just lost 1st 3 observations */
    lvwpd = -ln(vwdp);
    lvwdd = ln(vwdp);
    lvwdd = lvwdd[5:rows(lvwdd)]-lvwdd[1:rows(lvwdd)-4]+
          ln(arx[5:rows(arx)]);


    vwdpns = (vwret./vwretx - 1);
    lvwddns = ln(vwdpns[2:rows(vwdpns)])-ln(vwdpns[1:rows(vwdpns)-1])
               +ln(vwretx[2:rows(vwretx)]);

    vwret =  vwret[86:T];
    vwretx =  vwretx[86:T];
    vwdp =  vwdp[83:rows(vwdp)];
    lvwpd =  lvwpd[83:rows(lvwpd)];
    lvwdd = lvwdd[79:rows(lvwdd)];

    lvwddns = lvwddns[85:rows(lvwddns)];

    retp(ln(vwret),lvwpd,lvwdd,ln(vwretx),ln(tb),lndc,lndcpi,lvwddns,ln(gdiv));

endp;


proc(10) = aloaddat;
 /*
    returns campbell s&p500 and consumption data. All 1871-1994 with
    appropriate missing values.
    stocks are 1871-1994; consumption and its deflator are 1889-1982
 */

  local sdate,cdate,spind,div,spret,ppi,t,ncons,cpi, lnppi,lndc,
        lndppi,lndd,ir,rdgp,def,lnspret, lnir,lnsppd,x,rcons,lndelp,lndcdef;

  load x[] = \aaj\camrb\data\usaye.dat;
  x = reshape(x,rows(x)/4,4);
  x = x[2:rows(x),.] ; /* lop off initializing 100 for index, 0 for others*/
  sdate = x[.,1];   /* data is 1871 to 1993; */
  spind = x[.,2];   /* dec 31 s&p index */
  div   = x[.,3];   /* nominal dividends for the year */
  spret = x[.,4];   /* S&P total net return */

  load x[] = \aaj\camrb\data\usaym.dat;
  x = reshape(x,rows(x)/6,6);
  x = x[2:rows(x),.] ; /* lop off initializing 100 for index, 0 for others*/
  cdate = x[.,1];      /* data is 1871 to 1994 */
  ir  = x[1:rows(x)-1,2];        /* interest rate; -1 since 1 longer than s&p */
  ppi = x[1:rows(x)-1,3];        /* ppi level */
  rdgp = x[.,4];        /* missing */
  def  = x[.,5];       /* consumption deflator from GS */
  ncons = x[.,6];      /* per capita nominal consumption */

  cdate = selif(cdate,ncons .> 0);   /* eliminate missing values */
  def = selif(def,ncons .> 0);
  ncons = selif(ncons,ncons .> 0);
  rcons = ln(ncons./def);

  lndppi = ln(ppi[2:rows(ppi)]./ppi[1:rows(ppi)-1]);
  lndd = ln(div[2:rows(div)])-ln(div[1:rows(div)-1]);
  lndelp = ln(spind[2:rows(spind)])-ln(spind[1:rows(spind)-1]);

  lndc = rcons[2:rows(rcons)]-rcons[1:rows(rcons)-1];
  lndcdef = ln(def[2:rows(def)])-ln(def[1:rows(def)-1]);
  cdate = cdate[2:rows(cdate)];

  lnspret = ln(1+spret);
  lnspret = lnspret[2:rows(lnspret)];
  lnir = ln(1+ir/100);
  lnir = lnir[2:rows(lnir)];
  lnsppd = -ln(div./spind);
  lnsppd = lnsppd[2:rows(lnsppd)];
  sdate = sdate[2:rows(sdate)];

  retp(lnspret,lnsppd,lndd,lndelp,lnir,lndc,lndppi,lndcdef,sdate,cdate);
endp;

